<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:x="http://www.w3.org/TR/xpath"
                xmlns="http://software.in2p3.fr/xpath/execution-plan.xsd">
    <xsl:include href="xpath-serializer.xsl"/>

    <!-- todo: enable streaming with: /absolute/path[descendant{-or-self}] | /absolute/path
         => requires method XPathAbsolutePlan.playback() to forward events to XPathAbsoluteHandler
    -->
    <xsl:output method="xml" indent="yes"/>

    <xsl:template match="/x:XPath">
        <!-- relative path or expression => can not stream all XML events -->
        <xsl:for-each select="descendant::x:RelativeLocationPath[not(ancestor::x:AbsoluteLocationPath)]">
            <xsl:message terminate="yes">Expect absolute path instead of relative path: <xsl:apply-templates select="."/></xsl:message>
        </xsl:for-each>
        <xsl:for-each select="descendant::x:*[contains(local-name(),'Expr') and not(self::x:PathExpr|self::x:UnionExpr)][not(ancestor::x:AbsoluteLocationPath)]">
            <xsl:message terminate="yes">Expect absolute path instead of expression: <xsl:apply-templates select="."/></xsl:message>
        </xsl:for-each>

        <XPath>
            <!-- [ ... descendant{-or-self} ... ] -->
            <xsl:variable name="future-events-in-predicate" select="descendant::x:Predicate/descendant::x:*
                    [self::x:NameStep[starts-with(@axis,'descendant') or @axis='child']|self::x:TextNodeStep|self::x:CommentNodeStep]
                    [not(ancestor::x:RelativeLocationPath/preceding-sibling::x:FilterExpr/x:VariableReference)]"/>
            <!-- build full DOM tree if path contains one of the following expression, unless if relative to a $variable:
                position() or last()
                [ \d ]
                [ ... /absolute/path ... ]
                preceding{-sibling} | following{-sibling}
                ancestor{-or-self}  | parent
                [ ... ancestor{-or-self}/.../descendant{-or-self}) ... ]
                $future-events and count(/absolute/path)>1
            -->
            <xsl:choose>
                <xsl:when test="
                (   descendant::x:Function[@s1='position' or @s1='last']
                  | descendant::x:Predicate[x:PathExpr/x:FilterExpr/x:Number/@i][count(*)=1][count(*/*)=1][count(*/*/*)=1]
                  | descendant::x:Predicate/descendant::x:AbsoluteLocationPath
                  | descendant::x:NameStep[starts-with(@axis,'preceding') or starts-with(@axis,'following')]
                  | descendant::x:NameStep[starts-with(@axis,'ancestor') or @axis='parent'][not(ancestor::x:Predicate)]
                  | descendant::x:Predicate/descendant::x:NameStep[starts-with(@axis,'ancestor') or @axis='parent']/following-sibling::x:*
                    [self::x:NameStep[starts-with(@axis,'descendant') or @axis='child']|self::x:TextNodeStep|self::x:CommentNodeStep]
                )
                [not(ancestor::x:RelativeLocationPath/preceding-sibling::x:FilterExpr/x:VariableReference)]
                or ($future-events-in-predicate and count(//x:AbsoluteLocationPath)>1)">
                    <!-- build full DOM tree -->
                    <tree depth="1" localName="*">
                        <xsl:attribute name="nodes">
                            <xsl:apply-templates select="x:*"/>
                        </xsl:attribute>
                    </tree>
                </xsl:when>
                <xsl:when test="$future-events-in-predicate">
                    <!-- build partial DOM subtrees -->
                    <xsl:for-each select="$future-events-in-predicate/ancestor::x:AbsoluteLocationPath/child::x:NameStep
                    [not(preceding-sibling::x:NameStep/child::x:Predicate/descendant::x:*
                        [self::x:NameStep[starts-with(@axis,'descendant') or @axis='child'] | self::x:TextNodeStep | self::x:CommentNodeStep]
                        [not(ancestor::x:RelativeLocationPath/preceding-sibling::x:FilterExpr/x:VariableReference)]
                    )]
                    [last()]">
                        <tree>
                            <xsl:attribute name="nodes">
                                <!-- relative path to selected nodes -->
                                <xsl:choose>
                                    <xsl:when test="(self::x:NameStep | preceding-sibling::x:NameStep)[@axis='descendant' or @axis='descendant-or-self']">
                                        <xsl:text>descendant-or-self::</xsl:text>
                                    </xsl:when>
                                    <xsl:otherwise>self::</xsl:otherwise>
                                </xsl:choose>
                                <xsl:if test="@prefix"><xsl:value-of select="@prefix"/>:</xsl:if>
                                <xsl:value-of select="@localName"/>
                                <!-- predicates on selected nodes -->
                                <xsl:apply-templates select="child::x:Predicate | following-sibling::x:*"/>
                            </xsl:attribute>
                            <xsl:apply-templates mode="attributes" select="."/>
                            <xsl:apply-templates mode="predicates" select="."/>
                        </tree>
                    </xsl:for-each>
                </xsl:when>
                <xsl:otherwise>
                    <!-- select leaf node if it exists, else select current element  -->
                    <xsl:for-each select="descendant::x:AbsoluteLocationPath/x:NameStep[@axis!='attribute'][last()]">
                        <element>
                            <xsl:apply-templates mode="attributes" select="."/>
                            <xsl:apply-templates mode="predicates" select="."/>
                            <xsl:apply-templates mode="leaf" select="following-sibling::x:*[1]"/>
                        </element>
                    </xsl:for-each>
                </xsl:otherwise>
            </xsl:choose>
        </XPath>
    </xsl:template>

    <xsl:template match="x:NameStep" mode="attributes">
        <!-- for optimizations:
            * current depth != alternative depth => ignore this alternative.
            * current depth > max depth => do not create DOM4J element.
        -->
        <xsl:attribute name="depth">
            <xsl:choose>
                <xsl:when test="(preceding-sibling::x:NameStep|self::x:NameStep)[starts-with(@axis,'descendant')]">999</xsl:when>
                <xsl:otherwise><xsl:value-of select="count(preceding-sibling::x:NameStep|self::x:NameStep)"/></xsl:otherwise>
            </xsl:choose>
        </xsl:attribute>
        <xsl:for-each select="@prefix">
            <xsl:attribute name="namespace"><xsl:value-of select="."/></xsl:attribute>
        </xsl:for-each>
        <xsl:copy-of select="@localName"/>
    </xsl:template>

    <xsl:template match="x:NameStep[@axis='attribute']" mode="leaf">
        <attribute>
            <xsl:for-each select="@prefix">
                <xsl:attribute name="namespace"><xsl:value-of select="."/></xsl:attribute>
            </xsl:for-each>
            <xsl:copy-of select="@localName"/>
            <xsl:apply-templates select="x:Predicate" mode="leaf"/>
        </attribute>
    </xsl:template>
    <xsl:template match="x:TextNodeStep" mode="leaf">
        <text>
            <xsl:apply-templates select="x:Predicate" mode="leaf"/>
        </text>
    </xsl:template>
    <xsl:template match="x:CommentNodeStep" mode="leaf">
        <comment>
            <xsl:apply-templates select="x:Predicate" mode="leaf"/>
        </comment>
    </xsl:template>
    <xsl:template match="x:Predicate" mode="leaf">
        <predicate>
            <xsl:apply-templates select="x:*[last()]"/>
        </predicate>
    </xsl:template>

    <xsl:template match="x:NameStep" mode="predicates">
        <!-- add predicates based on current node -->
        <xsl:for-each select="child::x:Predicate[not(descendant::x:*
                [self::x:NameStep[starts-with(@axis,'descendant') or @axis='child'] | self::x:TextNodeStep | self::x:CommentNodeStep]
                [not(ancestor::x:RelativeLocationPath/preceding-sibling::x:FilterExpr/x:VariableReference)])]">
            <predicate>
                <xsl:apply-templates select="x:*[last()]"/>
            </predicate>
        </xsl:for-each>

        <!-- add predicate based on past nodes -->
        <xsl:if test="preceding-sibling::x:NameStep or @axis='child'">
            <predicate>
                <!-- relative path to candidate node -->
                <xsl:text>self::</xsl:text>
                <xsl:if test="@prefix"><xsl:value-of select="@prefix"/>:</xsl:if>
                <xsl:value-of select="@localName"/>
                <!-- predicates on candidate nodes -->
                <xsl:apply-templates select="preceding-sibling::x:NameStep">
                    <xsl:with-param name="reverse">true</xsl:with-param>
                    <xsl:sort select="position()" order="descending" data-type="number"/>
                </xsl:apply-templates>
                <xsl:if test="parent::x:*/x:NameStep[1][@axis='child']">[not(parent::*)]</xsl:if>
            </predicate>
        </xsl:if>
    </xsl:template>
</xsl:stylesheet>
