<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:xalan="http://xml.apache.org/xalan"
                xmlns:exslt="http://exslt.org/common"
                xmlns:cfg="http://software.in2p3.fr/lavoisier/config.xsd"
                xmlns:r="http://software.in2p3.fr/lavoisier/rendering.xsd"
                exclude-result-prefixes="xalan exslt cfg">
    <xsl:output method="xml" indent="yes" xalan:indent-amount="4"/>

    <xsl:variable name="xsl">http://www.w3.org/1999/XSL/Transform</xsl:variable>

    <xsl:template match="/cfg:pre-renderers">
        <xsl:element name="xsl:stylesheet" namespace="{$xsl}"><xsl:attribute name="version">1.0</xsl:attribute>
            <xsl:comment> generated by template-rowColumn.xsl </xsl:comment>
            <xsl:element name="xsl:output" namespace="{$xsl}"><xsl:attribute name="method">xml</xsl:attribute></xsl:element>
            <xsl:element name="xsl:template" namespace="{$xsl}"><xsl:attribute name="match">/</xsl:attribute>
                <!-- prefixes-to-namespaces mapping -->
                <xsl:for-each select="cfg:namespace">
                    <xsl:variable name="mapping">
                        <xsl:element name="{@prefix}:mapping" namespace="{@uri}"/>
                    </xsl:variable>
                    <xsl:copy-of select="exslt:node-set($mapping)/*/namespace::*[name()=current()/@prefix]"/>
                </xsl:for-each>

                <!-- rendering -->
                <r:rendering>
                    <!-- title -->
                    <xsl:for-each select="cfg:title">
                        <xsl:attribute name="title">{<xsl:value-of select="text()"/>}</xsl:attribute>
                    </xsl:for-each>

                    <!-- fields -->
                    <xsl:if test="cfg:field">
                        <r:fields>
                            <xsl:for-each select="cfg:field">
                                <r:field><xsl:copy-of select="@link"/>
                                    <xsl:attribute name="label">
                                        <xsl:choose>
                                            <xsl:when test="@label"><xsl:value-of select="@label"/></xsl:when>
                                            <xsl:otherwise>{<xsl:apply-templates select="text()"/>}</xsl:otherwise>
                                        </xsl:choose>
                                    </xsl:attribute>
                                    <xsl:element name="xsl:value-of" namespace="{$xsl}">
                                        <xsl:attribute name="select"><xsl:value-of select="text()"/></xsl:attribute>
                                    </xsl:element>
                                </r:field>
                            </xsl:for-each>
                        </r:fields>
                    </xsl:if>

                    <!-- rows -->
                    <xsl:if test="cfg:row">
                        <r:rows>
                            <xsl:for-each select="cfg:row">
                                <r:column_labels>
                                    <xsl:for-each select="cfg:column[not(@hidden='true')]">
                                        <r:column_label><xsl:copy-of select="@unit"/>
                                            <xsl:choose>
                                                <xsl:when test="@label"><xsl:value-of select="@label"/></xsl:when>
                                                <xsl:otherwise>
                                                    <xsl:element name="xsl:value-of" namespace="{$xsl}">
                                                        <xsl:attribute name="select">
                                                            <xsl:apply-templates select="text()"/>
                                                        </xsl:attribute>
                                                    </xsl:element>
                                                </xsl:otherwise>
                                            </xsl:choose>
                                        </r:column_label>
                                    </xsl:for-each>
                                </r:column_labels>
                                <xsl:element name="xsl:for-each" namespace="{$xsl}">
                                    <xsl:attribute name="select"><xsl:value-of select="@foreach"/></xsl:attribute>
                                    <xsl:for-each select="cfg:column[@order]">
                                        <xsl:element name="xsl:sort" namespace="{$xsl}">
                                            <xsl:attribute name="select"><xsl:value-of select="text()"/></xsl:attribute>
                                            <xsl:attribute name="order"><xsl:value-of select="@order"/></xsl:attribute>
                                            <xsl:if test="@unit">
                                                <xsl:attribute name="data-type">number</xsl:attribute>
                                            </xsl:if>
                                        </xsl:element>
                                    </xsl:for-each>
                                    <r:row>
                                        <xsl:for-each select="cfg:column[not(@hidden='true')]">
                                            <r:column><xsl:copy-of select="@link"/>
                                                <xsl:variable name="xpath" select="normalize-space(text())"/>
                                                <xsl:variable name="len" select="string-length($xpath)"/>
                                                <xsl:variable name="isString" select="
                                                    starts-with($xpath,'&quot;') or starts-with($xpath,&quot;'&quot;)"/>
                                                <xsl:variable name="isFunction" select="
                                                    substring($xpath,$len) = ')'
                                                        and substring($xpath,$len - 5) != 'text()'
                                                        and substring($xpath,$len - 8) != 'comment()'"/>
                                                <xsl:choose>
                                                    <xsl:when test="$isString or $isFunction">
                                                        <xsl:element name="xsl:value-of" namespace="{$xsl}">
                                                            <xsl:attribute name="select"><xsl:value-of select="$xpath"/></xsl:attribute>
                                                        </xsl:element>
                                                    </xsl:when>
                                                    <xsl:otherwise>
                                                        <xsl:element name="xsl:for-each" namespace="{$xsl}">
                                                            <xsl:attribute name="select"><xsl:value-of select="$xpath"/></xsl:attribute>
                                                            <xsl:element name="xsl:if" namespace="{$xsl}"><xsl:attribute name="test">position() > 1</xsl:attribute>,</xsl:element>
                                                            <xsl:element name="xsl:value-of" namespace="{$xsl}"><xsl:attribute name="select">.</xsl:attribute></xsl:element>
                                                        </xsl:element>
                                                    </xsl:otherwise>
                                                </xsl:choose>
                                            </r:column>
                                        </xsl:for-each>
                                    </r:row>
                                </xsl:element>
                            </xsl:for-each>
                        </r:rows>
                    </xsl:if>
                </r:rendering>
            </xsl:element>
        </xsl:element>
    </xsl:template>

    <xsl:template match="text()">
        <xsl:variable name="xpath_no_text">
            <xsl:variable name="ends_length" select="string-length(.)-string-length('/text()')+1"/>
            <xsl:choose>
                <xsl:when test="substring(.,$ends_length) = '/text()'">
                    <xsl:value-of select="substring(.,0,$ends_length)"/>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:value-of select="."/>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:variable>
        <xsl:variable name="xpath">
            <xsl:if test="not(starts-with($xpath_no_text,'/')) and parent::cfg:column/parent::cfg:row/@foreach">
                <xsl:value-of select="parent::cfg:column/parent::cfg:row/@foreach"/>/<xsl:text/>
            </xsl:if>
            <xsl:value-of select="$xpath_no_text"/>
        </xsl:variable>
        <xsl:text>local-name(</xsl:text>
        <xsl:value-of select="$xpath"/>
        <xsl:if test="not(starts-with($xpath,'/')) or contains(substring-after($xpath,'/'),'/')">[1]</xsl:if>
        <xsl:text>)</xsl:text>
    </xsl:template>
</xsl:stylesheet>