<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:xalan="http://xml.apache.org/xalan"
                xmlns:render="http://software.in2p3.fr/lavoisier/rendering.xsd">
    <xsl:output method="xml" indent="yes" xalan:indent-amount="4"/>
    <xsl:strip-space elements="*"/><!-- prevents text() function to ignore the text nodes other than 1st one -->
    <xsl:param name="view">Default rendering</xsl:param>

    <xsl:variable name="lower">abcdefghijklmnopqrstuvwxyz</xsl:variable>
    <xsl:variable name="UPPER">ABCDEFGHIJKLMNOPQRSTUVWXYZ</xsl:variable>

    <xsl:variable name="map2">map2</xsl:variable>
    <xsl:key name="map2"
             match="/*/*"
             use="concat('{',namespace-uri(),'}',local-name())"/>

    <xsl:variable name="hasContent">hasContent</xsl:variable>
    <xsl:key name="hasContent"
             match="/*/*[@*|text()|*/text()]"
             use="concat('{',namespace-uri(),'}',local-name())"/>

    <xsl:variable name="map3">map3</xsl:variable>
    <xsl:key name="map3"
             match="/*/*[key('map2',concat('{',namespace-uri(),'}',local-name()))[2]]/*"
             use="concat('{',namespace-uri(parent::*),'}',local-name(parent::*),'{',namespace-uri(),'}',local-name())"/>

    <xsl:variable name="isMulti">isMulti</xsl:variable>
    <xsl:key name="isMulti"
             match="/*/*[key('map2',concat('{',namespace-uri(),'}',local-name()))[2]]
                    /*[namespace-uri(following-sibling::*)=namespace-uri() and local-name(following-sibling::*)=local-name()]"
             use="concat('{',namespace-uri(parent::*),'}',local-name(parent::*),'{',namespace-uri(),'}',local-name())"/>

    <xsl:template match="/">
        <render:rendering title="{$view}">
            <xsl:variable name="level1_mono" select="/*"/>
            <xsl:variable name="level2_mono" select="/*/*[not(key($map2,concat('{',namespace-uri(),'}',local-name()))[2])]"/>
            <xsl:variable name="fields" select="($level1_mono | $level2_mono)[@*|text()]"/>
            <xsl:if test="$fields">
                <render:fields>
                    <xsl:apply-templates select="$fields | $fields/@*" mode="field"/>
                </render:fields>
            </xsl:if>

            <xsl:variable name="level2_multi" select="/*/*[key($map2,concat('{',namespace-uri(.),'}',local-name(.)))[2]]
                                                          [key($hasContent,concat('{',namespace-uri(.),'}',local-name(.)))[1]]"/>
            <xsl:variable name="level2_text" select="
                $level2_multi[generate-id()=generate-id(key($map2,concat('{',namespace-uri(),'}',local-name()))[text()][1])]
            "/>

            <xsl:variable name="level3_mono" select="
                $level2_multi/*[not(key($isMulti,concat('{',namespace-uri(parent::*),'}',local-name(parent::*),'{',namespace-uri(),'}',local-name()))[1])]
            "/>
            <xsl:variable name="level3_text" select="
                $level3_mono[generate-id()=generate-id(key($map3,concat('{',namespace-uri(parent::*),'}',local-name(parent::*),'{',namespace-uri(),'}',local-name()))[text()][1])]
            "/>
            <render:rows>
                <render:column_labels>
                    <render:column_label>Position</render:column_label>
                    <xsl:apply-templates select="$level2_multi[1]/@*" mode="column_label"/>
                    <xsl:apply-templates select="$level2_text" mode="column_label"/>
                    <xsl:apply-templates select="$level3_text" mode="column_label"/>
                </render:column_labels>
                <xsl:for-each select="$level2_multi">
                    <xsl:variable name="row" select="."/>
                    <render:row>
				        <xsl:variable name="href">
				        	<xsl:text>/lavoisier/</xsl:text>
				        	<xsl:value-of select="$view"/>
			        		<xsl:for-each select="ancestor-or-self::*">
                                <xsl:text/>/*[local-name()='<xsl:value-of select="local-name()"/>']<xsl:text/>
							</xsl:for-each>
							<xsl:text>[</xsl:text>
							<xsl:value-of select="position()"/>
							<xsl:text>]</xsl:text>
				        </xsl:variable>
                        <render:column link="{$href}">
				        	<xsl:value-of select="position()"/>
                        </render:column>
                        <xsl:apply-templates select="$level2_multi[1]/@*" mode="column">
                            <xsl:with-param name="row" select="$row"/>
                        </xsl:apply-templates>
                        <xsl:apply-templates select="$level2_text" mode="column">
                            <xsl:with-param name="row" select="$row"/>
                        </xsl:apply-templates>
                        <xsl:for-each select="$level3_text">
                            <xsl:variable name="current" select="local-name()"/>
                            <xsl:apply-templates select="." mode="column">
                                <xsl:with-param name="row" select="$row/*[local-name()=$current]"/>
                            </xsl:apply-templates>
                        </xsl:for-each>
                    </render:row>
                </xsl:for-each>
            </render:rows>
        </render:rendering>
    </xsl:template>

    <xsl:template match="/*" mode="field">
        <render:field label="{local-name()}">
            <xsl:value-of select="normalize-space(text())"/>
        </render:field>
    </xsl:template>
    <xsl:template match="/*/*" mode="field">
        <render:field label="{local-name()}">
            <xsl:value-of select="normalize-space(text())"/>
        </render:field>
    </xsl:template>
    <xsl:template match="/*/@*" mode="field">
        <render:field label="{local-name()}">
            <xsl:value-of select="."/>
        </render:field>
    </xsl:template>
    <xsl:template match="/*/*/@*" mode="field">
        <render:field label="{concat(local-name(parent::*), translate(substring(local-name(),1,1),$lower,$UPPER), substring(local-name(),2))}">
            <xsl:value-of select="."/>
        </render:field>
    </xsl:template>

    <xsl:template match="/*/*" mode="column_label">
        <render:column_label>
            <xsl:value-of select="local-name()"/>
        </render:column_label>
    </xsl:template>
    <xsl:template match="/*/*/*" mode="column_label">
        <render:column_label>
            <xsl:value-of select="concat(local-name(parent::*),'_',local-name())"/>
        </render:column_label>
    </xsl:template>
    <xsl:template match="/*/*/@*" mode="column_label">
        <render:column_label>
            <xsl:value-of select="concat(local-name(parent::*), translate(substring(local-name(),1,1),$lower,$UPPER), substring(local-name(),2))"/>
        </render:column_label>
    </xsl:template>

    <xsl:template match="/*/*" mode="column">
        <xsl:param name="row"/>
        <xsl:variable name="href">
        	<xsl:text>/lavoisier/</xsl:text>
        	<xsl:value-of select="$view"/>
      		<xsl:for-each select="ancestor-or-self::*">
                  <xsl:text/>/*[local-name()='<xsl:value-of select="local-name()"/>']<xsl:text/>
			</xsl:for-each>
            <xsl:text>[text()='</xsl:text>
            <xsl:value-of select="$row/text()"/>
            <xsl:text>']</xsl:text>
        </xsl:variable>
        <render:column link="{$href}">
            <xsl:value-of select="normalize-space($row[namespace-uri()=namespace-uri(current()) and local-name()=local-name(current())]/text())"/>
        </render:column>
    </xsl:template>
    <xsl:template match="/*/*/*" mode="column">
        <xsl:param name="row"/>
        <xsl:variable name="href">
        	<xsl:text>/lavoisier/</xsl:text>
        	<xsl:value-of select="$view"/>
      		<xsl:for-each select="ancestor-or-self::*">
                  <xsl:text/>/*[local-name()='<xsl:value-of select="local-name()"/>']<xsl:text/>
			</xsl:for-each>
            <xsl:text>[text()='</xsl:text>
            <xsl:value-of select="$row/text()"/>
            <xsl:text>']</xsl:text>
        </xsl:variable>
        <render:column link="{$href}">
            <xsl:value-of select="normalize-space($row[namespace-uri()=namespace-uri(current()) and local-name()=local-name(current())]/text())"/>
        </render:column>
    </xsl:template>
    <xsl:template match="/*/*/@*" mode="column">
        <xsl:param name="row"/>
        <xsl:variable name="href">
        	<xsl:text>/lavoisier/</xsl:text>
        	<xsl:value-of select="$view"/>
      		<xsl:for-each select="ancestor-or-self::*">
                  <xsl:text/>/*[local-name()='<xsl:value-of select="local-name()"/>']<xsl:text/>
			</xsl:for-each>
			<xsl:text>[@</xsl:text>
			<xsl:value-of select="local-name()"/>
			<xsl:text>='</xsl:text>
          	<xsl:value-of select="$row/@*"/>
			<xsl:text>']</xsl:text>
        </xsl:variable>
        <render:column link="{$href}">
            <xsl:value-of select="$row/@*[local-name()=local-name(current())]"/>
        </render:column>
    </xsl:template>
</xsl:stylesheet>